from functional_parsing_library.combinators.sequence.separated_by import nothing
from functional_parsing_library.examples.person import const
from functional_parsing_library.parser import Parser, S, TokenStream


def n_times(n: int, parser: Parser[TokenStream, S]) -> Parser[TokenStream, list[S]]:
    if n == 0:
        return const([]) * nothing
    return (lambda t, ts: [t, *ts]) * parser & n_times(n - 1, parser)



from typing import TypeVar, Any

from functional_parsing_library import Parser, CouldNotParse
from functional_parsing_library.parser import ParseResults

Token = TypeVar('Token')


def token_of_type(token: type[Token]) -> Parser[list[Any], Token]:
    def _parse_token(to_parse: list[Token]) -> ParseResults[list[Token], Token] | CouldNotParse:
        if len(to_parse) == 0:
            return CouldNotParse()
        if isinstance(to_parse[0], token):
            return ParseResults(
                result=to_parse[0],
                remainder=to_parse[1:],
            )
        return CouldNotParse()

    return Parser(_parse_token)



from typing import Callable

from functional_parsing_library.parser import Parser, T, S, ParseResults, CouldNotParse, TokenStream


def bind(parser: Parser[TokenStream, T], parser_function: Callable[[T], Parser[TokenStream, S]]) -> Parser[TokenStream, S]:
    def _parser_function(to_parse: TokenStream) -> ParseResults[TokenStream, S] | CouldNotParse:
        parse_results = parser(to_parse)
        if isinstance(parse_results, CouldNotParse):
            return parse_results
        s_parser = parser_function(parse_results.result)
        return s_parser(parse_results.remainder)

    return Parser(_parser_function)



from .run_parser import run_parser as run_parser, NotFullyParsed as NotFullyParsed, ParsingError as ParsingError
from .strings import *
from .combinators import *
from .parser import CouldNotParse as CouldNotParse, Parser as Parser
from .integer.integer import integer as integer
